"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.EVENT_INVOKE = void 0;

var _assert = _interopRequireDefault(require("assert"));

var _events = _interopRequireDefault(require("events"));

var _backends2 = require("./backends");

var _getCommands = _interopRequireDefault(require("./backends/get-commands"));

var _stackFrame = require("./stack-frame");

require("./symbol-async-iterator");

var _statisticEntry = _interopRequireDefault(require("./statistic-entry"));

var _utils = require("./utils");

var _yieldToEventLoop = _interopRequireDefault(require("./yield-to-event-loop"));

var _getBackends = _interopRequireDefault(require("./get-backends"));

var _sampler = _interopRequireWildcard(require("./sampler"));

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _wrapAsyncGenerator(fn) { return function () { return new _AsyncGenerator(fn.apply(this, arguments)); }; }

function _AsyncGenerator(gen) { var front, back; function send(key, arg) { return new Promise(function (resolve, reject) { var request = { key: key, arg: arg, resolve: resolve, reject: reject, next: null }; if (back) { back = back.next = request; } else { front = back = request; resume(key, arg); } }); } function resume(key, arg) { try { var result = gen[key](arg); var value = result.value; var wrappedAwait = value instanceof _AwaitValue; Promise.resolve(wrappedAwait ? value.wrapped : value).then(function (arg) { if (wrappedAwait) { resume("next", arg); return; } settle(result.done ? "return" : "normal", arg); }, function (err) { resume("throw", err); }); } catch (err) { settle("throw", err); } } function settle(type, value) { switch (type) { case "return": front.resolve({ value: value, done: true }); break; case "throw": front.reject(value); break; default: front.resolve({ value: value, done: false }); break; } front = front.next; if (front) { resume(front.key, front.arg); } else { back = null; } } this._invoke = send; if (typeof gen.return !== "function") { this.return = undefined; } }

if (typeof Symbol === "function" && Symbol.asyncIterator) { _AsyncGenerator.prototype[Symbol.asyncIterator] = function () { return this; }; }

_AsyncGenerator.prototype.next = function (arg) { return this._invoke("next", arg); };

_AsyncGenerator.prototype.throw = function (arg) { return this._invoke("throw", arg); };

_AsyncGenerator.prototype.return = function (arg) { return this._invoke("return", arg); };

function _awaitAsyncGenerator(value) { return new _AwaitValue(value); }

function _AwaitValue(value) { this.wrapped = value; }

function _asyncIterator(iterable) { var method; if (typeof Symbol === "function") { if (Symbol.asyncIterator) { method = iterable[Symbol.asyncIterator]; if (method != null) return method.call(iterable); } if (Symbol.iterator) { method = iterable[Symbol.iterator]; if (method != null) return method.call(iterable); } } throw new TypeError("Object is not async iterable"); }

function _asyncGeneratorDelegate(inner, awaitWrap) { var iter = {}, waiting = false; function pump(key, value) { waiting = true; value = new Promise(function (resolve) { resolve(inner[key](value)); }); return { done: false, value: awaitWrap(value) }; } ; if (typeof Symbol === "function" && Symbol.iterator) { iter[Symbol.iterator] = function () { return this; }; } iter.next = function (value) { if (waiting) { waiting = false; return value; } return pump("next", value); }; if (typeof inner.throw === "function") { iter.throw = function (value) { if (waiting) { waiting = false; throw value; } return pump("throw", value); }; } if (typeof inner.return === "function") { iter.return = function (value) { return pump("return", value); }; } return iter; }

const EVENT_INVOKE = 'invoke';
exports.EVENT_INVOKE = EVENT_INVOKE;

class Context extends _events.default {
  constructor({
    statistics = _sampler.NOOP
  } = {}) {
    super();
    const keys = new Map();

    this._keyFor = backend => {
      const key = keys.get(backend);
      if (key) return key;
      const newKey = `_backend.${keys.size + 1}`;
      keys.set(backend, newKey);
      return newKey;
    };

    this._commands = {};
    this._sampler = (0, _sampler.default)(statistics);

    if (statistics !== _sampler.NOOP) {
      this._statistics = new Map();
    }

    this.use((0, _backends2.standard)());
  }

  use(...backends) {
    (0, _assert.default)(backends.length > 0, 'Must provide at least one backend');
    const commands = (0, _getCommands.default)(backends[0]);
    (0, _assert.default)(commands.filter(op => this._commands[op]).length === 0, `Commands already bound: ${commands.filter(op => this._commands[op]).join(', ')}`);

    for (const op of commands) {
      if (this._statistics) {
        this._statistics.set(op, new _statisticEntry.default());
      }

      this._commands[op] = backends;
    }

    backends.forEach(backend => {
      const events = (0, _backends2.getListeningEvents)(backend);
      events.forEach(event => {
        const key = this._keyFor(backend);

        const fn = backend[(0, _backends2.listenerName)(event)];
        this.on(event, function listener(...args) {
          const context = this[key] || backend;
          return fn.apply(context, args);
        });
      });
    });
    return this;
  }

  statistics() {
    return this._statistics;
  }

  _statisticEntry(id) {
    if (!this._statistics) return this._sampler;

    if (this._statistics.has(id)) {
      const entry = this._statistics.get(id);

      return fn => this._sampler(fn, entry);
    }

    const entry = new _statisticEntry.default();

    this._statistics.set(id, entry);

    return fn => this._sampler(fn, entry);
  }

  _command(value, ...backends) {
    const {
      type,
      payload,
      stackFrame
    } = value;

    const _backends = backends.length === 0 ? this._commands[type] : backends;

    (0, _assert.default)(_backends, `No backends for command ${type}`);
    (0, _assert.default)((0, _utils.isString)(type), 'command.type must be string');
    (0, _assert.default)(type[0] !== '_', `command.type "${type}" must not start with _`);
    this.emit(EVENT_INVOKE, type, payload);

    const createNext = index => {
      const fn = index < _backends.length && _backends[index][type];

      if (!fn) {
        return null;
      }

      (0, _assert.default)(fn.length <= 2, `command.type "${type}" must take max two arguments (not ${fn.length})`);
      const backend = _backends[index];

      const key = this._keyFor(backend);

      const call = (...args) => {
        const entry = this._statisticEntry(`${type}.${index}`);

        const context = this[key] || backend;
        return entry(() => fn.call(context, ...args));
      };

      if (fn.length === 1) {
        return () => call(payload);
      }

      return () => {
        const next = createNext(index + 1);

        const withContext = subcontext => subcontext ? Object.create(this, {
          [key]: {
            value: subcontext
          }
        }) : this;

        const execute = (_value, subcontext) => {
          const _subbackends = (0, _getBackends.default)(_value);

          if (_subbackends.length) {
            return withContext(subcontext)._command(_value, ..._subbackends);
          }

          return withContext(subcontext).execute(_value);
        };

        const iterate = (_value, subcontext) => withContext(subcontext).iterate(_value);

        const commandContext = next ? {
          execute,
          iterate,
          next
        } : {
          execute,
          iterate
        };
        return call(payload, commandContext);
      };
    };

    const entry = this._statisticEntry(type);

    return (0, _stackFrame.withStackFrame)(stackFrame, () => entry(createNext(0)));
  }

  async execute(value) {
    const v = await this.iterate(value).next();

    if (!v.done) {
      throw new Error(`yielding literal values inside execute is not allowed: ${v.value}`);
    }

    return v.value;
  }

  iterate(value) {
    var _this = this;

    return _wrapAsyncGenerator(function* () {
      if ((0, _utils.isCommand)(value)) {
        return _this._command(value);
      }

      const context = _this;

      const step =
      /*#__PURE__*/
      function () {
        var _step = _wrapAsyncGenerator(function* (v) {
          let nextValue;

          try {
            if ((0, _utils.isCommand)(v)) {
              nextValue = yield _awaitAsyncGenerator(context._command(v));
            } else {
              nextValue = yield v;
            }

            yield _awaitAsyncGenerator((0, _yieldToEventLoop.default)());
          } catch (e) {
            return value.throw(e);
          }

          return value.next(nextValue);
        });

        return function step(_x) {
          return _step.apply(this, arguments);
        };
      }();

      let v = yield _awaitAsyncGenerator(value.next());

      while (!v.done) {
        v = yield* _asyncGeneratorDelegate(_asyncIterator(step(v.value)), _awaitAsyncGenerator);
      }

      return v.value;
    })();
  }

}

exports.default = Context;