"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.DIFF = exports.ASYNC = exports.NOOP = void 0;

var _perf_hooks = require("perf_hooks");

var _asyncTracking = _interopRequireDefault(require("./async-tracking"));

var _utils = require("./utils");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const NOOP = 'noop';
exports.NOOP = NOOP;
const ASYNC = 'async';
exports.ASYNC = ASYNC;
const DIFF = 'diff';
exports.DIFF = DIFF;
const sampler = {
  [NOOP](fn) {
    return fn();
  },

  [ASYNC](fn, entry) {
    const executionContext = new _asyncTracking.default();
    const result = executionContext.run(fn);
    Promise.resolve(result).then(() => {
      entry.addSample(executionContext.duration + executionContext.blackTime);
    });
    return result;
  },

  [DIFF](fn, entry) {
    const start = _perf_hooks.performance.now();

    const done = () => entry.addSample(_perf_hooks.performance.now() - start);

    const result = fn();

    if ((0, _utils.isPromise)(result)) {
      return (0, _utils.promiseFinally)(result, done);
    }

    done();
    return result;
  }

};

var _default = id => {
  const result = sampler[id];
  if (!result) throw new Error(`Sampler '${id}' is unknown`);
  return result;
};

exports.default = _default;