"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.evaluateWithStackFrame = evaluateWithStackFrame;
exports.withStackFrame = exports.commandWithStackFrame = exports.commandCreatorForEnvironment = exports.StackFrame = void 0;

var _utils = require("./utils");

function _wrapAsyncGenerator(fn) { return function () { return new _AsyncGenerator(fn.apply(this, arguments)); }; }

function _AsyncGenerator(gen) { var front, back; function send(key, arg) { return new Promise(function (resolve, reject) { var request = { key: key, arg: arg, resolve: resolve, reject: reject, next: null }; if (back) { back = back.next = request; } else { front = back = request; resume(key, arg); } }); } function resume(key, arg) { try { var result = gen[key](arg); var value = result.value; var wrappedAwait = value instanceof _AwaitValue; Promise.resolve(wrappedAwait ? value.wrapped : value).then(function (arg) { if (wrappedAwait) { resume("next", arg); return; } settle(result.done ? "return" : "normal", arg); }, function (err) { resume("throw", err); }); } catch (err) { settle("throw", err); } } function settle(type, value) { switch (type) { case "return": front.resolve({ value: value, done: true }); break; case "throw": front.reject(value); break; default: front.resolve({ value: value, done: false }); break; } front = front.next; if (front) { resume(front.key, front.arg); } else { back = null; } } this._invoke = send; if (typeof gen.return !== "function") { this.return = undefined; } }

if (typeof Symbol === "function" && Symbol.asyncIterator) { _AsyncGenerator.prototype[Symbol.asyncIterator] = function () { return this; }; }

_AsyncGenerator.prototype.next = function (arg) { return this._invoke("next", arg); };

_AsyncGenerator.prototype.throw = function (arg) { return this._invoke("throw", arg); };

_AsyncGenerator.prototype.return = function (arg) { return this._invoke("return", arg); };

function _awaitAsyncGenerator(value) { return new _AwaitValue(value); }

function _AwaitValue(value) { this.wrapped = value; }

function _asyncIterator(iterable) { var method; if (typeof Symbol === "function") { if (Symbol.asyncIterator) { method = iterable[Symbol.asyncIterator]; if (method != null) return method.call(iterable); } if (Symbol.iterator) { method = iterable[Symbol.iterator]; if (method != null) return method.call(iterable); } } throw new TypeError("Object is not async iterable"); }

function _asyncGeneratorDelegate(inner, awaitWrap) { var iter = {}, waiting = false; function pump(key, value) { waiting = true; value = new Promise(function (resolve) { resolve(inner[key](value)); }); return { done: false, value: awaitWrap(value) }; } ; if (typeof Symbol === "function" && Symbol.iterator) { iter[Symbol.iterator] = function () { return this; }; } iter.next = function (value) { if (waiting) { waiting = false; return value; } return pump("next", value); }; if (typeof inner.throw === "function") { iter.throw = function (value) { if (waiting) { waiting = false; throw value; } return pump("throw", value); }; } if (typeof inner.return === "function") { iter.return = function (value) { return pump("return", value); }; } return iter; }

const BORDERS_STACK_PATTERN = /^.*\/borders\/lib\/.*$/;

const filterStack = stack => stack.filter(part => !BORDERS_STACK_PATTERN.test(part));

class StackFrame extends Error {
  constructor() {
    super();
    Error.captureStackTrace(this, StackFrame);
  }

  attachStack(err) {
    if (!err.__originalStack__) {
      Object.defineProperty(err, '__originalStack__', {
        configurable: true,
        enumerable: false,
        writable: false,
        value: err.stack
      });
    }

    const self = this;
    Object.defineProperty(err, 'stack', {
      configurable: true,
      enumerable: false,

      get() {
        const newStack = [...this.__originalStack__.split('\n'), 'From previous event:', ...self.stack.split('\n').slice(1)];
        const filteredStack = filterStack(newStack);
        return filteredStack.join('\n');
      }

    });
  }

}

exports.StackFrame = StackFrame;

const _commandWithStackFrame = commandCreator => (...args) => {
  const command = commandCreator(...args);
  command.stackFrame = new StackFrame();
  return command;
};

const commandCreatorForEnvironment = env => env === 'production' ? commandCreator => commandCreator : _commandWithStackFrame;

exports.commandCreatorForEnvironment = commandCreatorForEnvironment;
const commandWithStackFrame = commandCreatorForEnvironment(process.env.NODE_ENV);
exports.commandWithStackFrame = commandWithStackFrame;

const withStackFrame = (stackFrame, fn) => {
  if (!stackFrame) {
    return fn();
  }

  let result;

  try {
    result = fn();
  } catch (e) {
    stackFrame.attachStack(e);
    throw e;
  }

  if (!(0, _utils.isPromise)(result)) {
    return result;
  }

  return result.then(undefined, reason => {
    stackFrame.attachStack(reason);
    throw reason;
  });
};

exports.withStackFrame = withStackFrame;

function evaluateWithStackFrame(_x, _x2) {
  return _evaluateWithStackFrame.apply(this, arguments);
}

function _evaluateWithStackFrame() {
  _evaluateWithStackFrame = _wrapAsyncGenerator(function* (stackFrame, generator) {
    if (!stackFrame) return yield* _asyncGeneratorDelegate(_asyncIterator(generator), _awaitAsyncGenerator);

    try {
      return yield* _asyncGeneratorDelegate(_asyncIterator(generator), _awaitAsyncGenerator);
    } catch (e) {
      stackFrame.attachStack(e);
      throw e;
    }
  });
  return _evaluateWithStackFrame.apply(this, arguments);
}